# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class AviaryModelConfigV2(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'base_model_id': 'str',
        'model_id': 'str',
        'generation': 'object',
        'cloud_id': 'str',
        'project_id': 'str',
        'ha_job_id': 'str',
        'workspace_id': 'str',
        'model_metadata': 'object',
        'lora_mirror_config': 'object',
        'full_param_mirror_config': 'object'
    }

    attribute_map = {
        'base_model_id': 'base_model_id',
        'model_id': 'model_id',
        'generation': 'generation',
        'cloud_id': 'cloud_id',
        'project_id': 'project_id',
        'ha_job_id': 'ha_job_id',
        'workspace_id': 'workspace_id',
        'model_metadata': 'model_metadata',
        'lora_mirror_config': 'lora_mirror_config',
        'full_param_mirror_config': 'full_param_mirror_config'
    }

    def __init__(self, base_model_id=None, model_id=None, generation=None, cloud_id=None, project_id=None, ha_job_id=None, workspace_id=None, model_metadata=None, lora_mirror_config=None, full_param_mirror_config=None, local_vars_configuration=None):  # noqa: E501
        """AviaryModelConfigV2 - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._base_model_id = None
        self._model_id = None
        self._generation = None
        self._cloud_id = None
        self._project_id = None
        self._ha_job_id = None
        self._workspace_id = None
        self._model_metadata = None
        self._lora_mirror_config = None
        self._full_param_mirror_config = None
        self.discriminator = None

        self.base_model_id = base_model_id
        self.model_id = model_id
        self.generation = generation
        self.cloud_id = cloud_id
        if project_id is not None:
            self.project_id = project_id
        if ha_job_id is not None:
            self.ha_job_id = ha_job_id
        if workspace_id is not None:
            self.workspace_id = workspace_id
        if model_metadata is not None:
            self.model_metadata = model_metadata
        if lora_mirror_config is not None:
            self.lora_mirror_config = lora_mirror_config
        if full_param_mirror_config is not None:
            self.full_param_mirror_config = full_param_mirror_config

    @property
    def base_model_id(self):
        """Gets the base_model_id of this AviaryModelConfigV2.  # noqa: E501


        :return: The base_model_id of this AviaryModelConfigV2.  # noqa: E501
        :rtype: str
        """
        return self._base_model_id

    @base_model_id.setter
    def base_model_id(self, base_model_id):
        """Sets the base_model_id of this AviaryModelConfigV2.


        :param base_model_id: The base_model_id of this AviaryModelConfigV2.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and base_model_id is None:  # noqa: E501
            raise ValueError("Invalid value for `base_model_id`, must not be `None`")  # noqa: E501

        self._base_model_id = base_model_id

    @property
    def model_id(self):
        """Gets the model_id of this AviaryModelConfigV2.  # noqa: E501


        :return: The model_id of this AviaryModelConfigV2.  # noqa: E501
        :rtype: str
        """
        return self._model_id

    @model_id.setter
    def model_id(self, model_id):
        """Sets the model_id of this AviaryModelConfigV2.


        :param model_id: The model_id of this AviaryModelConfigV2.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and model_id is None:  # noqa: E501
            raise ValueError("Invalid value for `model_id`, must not be `None`")  # noqa: E501

        self._model_id = model_id

    @property
    def generation(self):
        """Gets the generation of this AviaryModelConfigV2.  # noqa: E501


        :return: The generation of this AviaryModelConfigV2.  # noqa: E501
        :rtype: object
        """
        return self._generation

    @generation.setter
    def generation(self, generation):
        """Sets the generation of this AviaryModelConfigV2.


        :param generation: The generation of this AviaryModelConfigV2.  # noqa: E501
        :type: object
        """
        if self.local_vars_configuration.client_side_validation and generation is None:  # noqa: E501
            raise ValueError("Invalid value for `generation`, must not be `None`")  # noqa: E501

        self._generation = generation

    @property
    def cloud_id(self):
        """Gets the cloud_id of this AviaryModelConfigV2.  # noqa: E501


        :return: The cloud_id of this AviaryModelConfigV2.  # noqa: E501
        :rtype: str
        """
        return self._cloud_id

    @cloud_id.setter
    def cloud_id(self, cloud_id):
        """Sets the cloud_id of this AviaryModelConfigV2.


        :param cloud_id: The cloud_id of this AviaryModelConfigV2.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and cloud_id is None:  # noqa: E501
            raise ValueError("Invalid value for `cloud_id`, must not be `None`")  # noqa: E501

        self._cloud_id = cloud_id

    @property
    def project_id(self):
        """Gets the project_id of this AviaryModelConfigV2.  # noqa: E501


        :return: The project_id of this AviaryModelConfigV2.  # noqa: E501
        :rtype: str
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """Sets the project_id of this AviaryModelConfigV2.


        :param project_id: The project_id of this AviaryModelConfigV2.  # noqa: E501
        :type: str
        """

        self._project_id = project_id

    @property
    def ha_job_id(self):
        """Gets the ha_job_id of this AviaryModelConfigV2.  # noqa: E501


        :return: The ha_job_id of this AviaryModelConfigV2.  # noqa: E501
        :rtype: str
        """
        return self._ha_job_id

    @ha_job_id.setter
    def ha_job_id(self, ha_job_id):
        """Sets the ha_job_id of this AviaryModelConfigV2.


        :param ha_job_id: The ha_job_id of this AviaryModelConfigV2.  # noqa: E501
        :type: str
        """

        self._ha_job_id = ha_job_id

    @property
    def workspace_id(self):
        """Gets the workspace_id of this AviaryModelConfigV2.  # noqa: E501


        :return: The workspace_id of this AviaryModelConfigV2.  # noqa: E501
        :rtype: str
        """
        return self._workspace_id

    @workspace_id.setter
    def workspace_id(self, workspace_id):
        """Sets the workspace_id of this AviaryModelConfigV2.


        :param workspace_id: The workspace_id of this AviaryModelConfigV2.  # noqa: E501
        :type: str
        """

        self._workspace_id = workspace_id

    @property
    def model_metadata(self):
        """Gets the model_metadata of this AviaryModelConfigV2.  # noqa: E501


        :return: The model_metadata of this AviaryModelConfigV2.  # noqa: E501
        :rtype: object
        """
        return self._model_metadata

    @model_metadata.setter
    def model_metadata(self, model_metadata):
        """Sets the model_metadata of this AviaryModelConfigV2.


        :param model_metadata: The model_metadata of this AviaryModelConfigV2.  # noqa: E501
        :type: object
        """

        self._model_metadata = model_metadata

    @property
    def lora_mirror_config(self):
        """Gets the lora_mirror_config of this AviaryModelConfigV2.  # noqa: E501


        :return: The lora_mirror_config of this AviaryModelConfigV2.  # noqa: E501
        :rtype: object
        """
        return self._lora_mirror_config

    @lora_mirror_config.setter
    def lora_mirror_config(self, lora_mirror_config):
        """Sets the lora_mirror_config of this AviaryModelConfigV2.


        :param lora_mirror_config: The lora_mirror_config of this AviaryModelConfigV2.  # noqa: E501
        :type: object
        """

        self._lora_mirror_config = lora_mirror_config

    @property
    def full_param_mirror_config(self):
        """Gets the full_param_mirror_config of this AviaryModelConfigV2.  # noqa: E501


        :return: The full_param_mirror_config of this AviaryModelConfigV2.  # noqa: E501
        :rtype: object
        """
        return self._full_param_mirror_config

    @full_param_mirror_config.setter
    def full_param_mirror_config(self, full_param_mirror_config):
        """Sets the full_param_mirror_config of this AviaryModelConfigV2.


        :param full_param_mirror_config: The full_param_mirror_config of this AviaryModelConfigV2.  # noqa: E501
        :type: object
        """

        self._full_param_mirror_config = full_param_mirror_config

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, AviaryModelConfigV2):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, AviaryModelConfigV2):
            return True

        return self.to_dict() != other.to_dict()
