# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class FinishFTJobRequestV2(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'ft_type': 'str',
        'model_config_version': 'str',
        'model_config': 'AviaryModelConfigV2'
    }

    attribute_map = {
        'ft_type': 'ft_type',
        'model_config_version': 'model_config_version',
        'model_config': 'model_config'
    }

    def __init__(self, ft_type=None, model_config_version=None, model_config=None, local_vars_configuration=None):  # noqa: E501
        """FinishFTJobRequestV2 - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._ft_type = None
        self._model_config_version = None
        self._model_config = None
        self.discriminator = None

        self.ft_type = ft_type
        self.model_config_version = model_config_version
        self.model_config = model_config

    @property
    def ft_type(self):
        """Gets the ft_type of this FinishFTJobRequestV2.  # noqa: E501


        :return: The ft_type of this FinishFTJobRequestV2.  # noqa: E501
        :rtype: str
        """
        return self._ft_type

    @ft_type.setter
    def ft_type(self, ft_type):
        """Sets the ft_type of this FinishFTJobRequestV2.


        :param ft_type: The ft_type of this FinishFTJobRequestV2.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and ft_type is None:  # noqa: E501
            raise ValueError("Invalid value for `ft_type`, must not be `None`")  # noqa: E501
        allowed_values = ["lora", "full"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and ft_type not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `ft_type` ({0}), must be one of {1}"  # noqa: E501
                .format(ft_type, allowed_values)
            )

        self._ft_type = ft_type

    @property
    def model_config_version(self):
        """Gets the model_config_version of this FinishFTJobRequestV2.  # noqa: E501

        The version of LLMForge  # noqa: E501

        :return: The model_config_version of this FinishFTJobRequestV2.  # noqa: E501
        :rtype: str
        """
        return self._model_config_version

    @model_config_version.setter
    def model_config_version(self, model_config_version):
        """Sets the model_config_version of this FinishFTJobRequestV2.

        The version of LLMForge  # noqa: E501

        :param model_config_version: The model_config_version of this FinishFTJobRequestV2.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and model_config_version is None:  # noqa: E501
            raise ValueError("Invalid value for `model_config_version`, must not be `None`")  # noqa: E501

        self._model_config_version = model_config_version

    @property
    def model_config(self):
        """Gets the model_config of this FinishFTJobRequestV2.  # noqa: E501


        :return: The model_config of this FinishFTJobRequestV2.  # noqa: E501
        :rtype: AviaryModelConfigV2
        """
        return self._model_config

    @model_config.setter
    def model_config(self, model_config):
        """Sets the model_config of this FinishFTJobRequestV2.


        :param model_config: The model_config of this FinishFTJobRequestV2.  # noqa: E501
        :type: AviaryModelConfigV2
        """
        if self.local_vars_configuration.client_side_validation and model_config is None:  # noqa: E501
            raise ValueError("Invalid value for `model_config`, must not be `None`")  # noqa: E501

        self._model_config = model_config

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, FinishFTJobRequestV2):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, FinishFTJobRequestV2):
            return True

        return self.to_dict() != other.to_dict()
